#!/bin/bash

#616732 Set umask to friendly value so that all files/directories created by configuration tool can be accessed
umask 0022

source /etc/sysconfig/outsystems
source $OUTSYSTEMS_HOME/common.sh
checkIsRoot "ConfigurationTool must be run as root"

if ! type -p log_warning_msg &> /dev/null ; then
        function log_warning_msg() {
                echo "[WARNING] $1"
        }
fi

function changeUserUmask() {
	eval UMASK_DIR="$(printf "~%q" "$1")"

	if [ ! -f $UMASK_DIR/.bash_profile ]; then
		cp /etc/skel/.bash_profile $UMASK_DIR/
		chown "$1":"$1" $UMASK_DIR/.bash_profile
	fi
	
	if grep -q 'umask' $UMASK_DIR/.bash_profile; then
		if grep -q 'umask 0027' $UMASK_DIR/.bash_profile; then
			return
		else
			log_warning_msg "'$1' user default umask changed to 0027"
		fi
	fi
	eval echo "umask 0027" >> $UMASK_DIR/.bash_profile
}

#616730 Allow application server user to read outystems user files
OUTSYSTEMS_USER=$(stat -c %U $OUTSYSTEMS_HOME)
changeUserUmask $OUTSYSTEMS_USER

if [ ! -e $SERVICESCONFIG ]; then
        $OUTSYSTEMS_HOME/serviceconfigurator.sh -all
fi
source $SERVICESCONFIG

# Source function library.
if [ -f /lib/lsb/init-functions ]; then
        . /lib/lsb/init-functions
else
        . /etc/rc.d/init.d/functions
fi

function isApplicationServerRestartNeeded(){
	
    ANSWER="n"	# We override the answer if needed.

    if [ "$NEW_VERSION" == "y" ]
    then
        ANSWER="y"
    fi

    if [ "$silentmode" == "y" ]
    then
        ANSWER="y"
    fi

    echo $ANSWER
}

function createTnsNames() {
if [ ! -e /etc/outsystems/tnsnames.ora ]; then
cat << _EOF_ > /etc/outsystems/tnsnames.ora
# This file contains the syntax information for 
# the entries to be put in any tnsnames.ora file
# The entries in this file are need based. 

# They are a collection of aliases for the addresses that 
# the listener(s) is(are) listening for a database or 
# several databases.

# The following is the general syntax for any entry in 
# a tnsnames.ora file. There could be several such entries 
# tailored to the user's needs.

<alias>= [ (DESCRIPTION_LIST =  # Optional depending on whether u have 
				# one or more descriptions
				# If there is just one description, unnecessary ]
	  (DESCRIPTION=
	    [ (SDU=2048) ]	# Optional, defaults to 2048
				# Can take values between 512 and 32K
	    [ (ADDRESS_LIST=    # Optional depending on whether u have
				# one or more addresses
				# If there is just one address, unnecessary ]
	      (ADDRESS=
		[ (COMMUNITY=<community_name>) ] 
		(PROTOCOL=tcp)
		(HOST=<hostname>)
		(PORT=<portnumber (1521 is a standard port used)>)
	      )
	      [ (ADDRESS=
		  (PROTOCOL=ipc)
		  (KEY=<ipckey (PNPKEY is a standard key used)>)	
		)
	      ]
	      [ (ADDRESS=
		  [ (COMMUNITY=<community_name>) ]
		  (PROTOCOL=decnet)
		  (NODE=<nodename>)
		  (OBJECT=<objectname>)
		)
	      ]
              ... # More addresses
	    [ ) ] # Optional depending on whether ADDRESS_LIST is used or not 
	    [ (CONNECT_DATA=
		(SID=<oracle_sid>)
		[ (GLOBAL_NAME=<global_database_name>) ]
	      )
	    ]
	    [ (SOURCE_ROUTE=yes) ]  
	  )
	  (DESCRIPTION=     
	    [ (SDU=2048) ]	# Optional, defaults to 2048
				# Can take values between 512 and 32K
	    [ (ADDRESS_LIST= ]	# Optional depending on whether u have more
				# than one address or not
				# If there is just one address, unnecessary
	      (ADDRESS
		[ (COMMUNITY=<community_name>) ]
		(PROTOCOL=tcp)
		(HOST=<hostname>)
		(PORT=<portnumber (1521 is a standard port used)>)
	      )
	      [ (ADDRESS=
		  (PROTOCOL=ipc)
		  (KEY=<ipckey (PNPKEY is a standard key used)>)
	         )
	      ]
	      ... 		# More addresses
	    [ ) ] 		# Optional depending on whether ADDRESS_LIST  
				# is being used
	    [ (CONNECT_DATA=
		(SID=<oracle_sid>)
		[ (GLOBAL_NAME=<global_database_name>) ]
	      )
	    ]
	    [ (SOURCE_ROUTE=yes) ]
	  )
	  [ (CONNECT_DATA=
	      (SID=<oracle_sid>)
	      [ (GLOBAL_NAME=<global_database_name>) ]
	    )
	  ]
	  ...   # More descriptions 
	[ ) ]	# Optional depending on whether DESCRIPTION_LIST is used or not
_EOF_
fi
}

createTnsNames

cd $OUTSYSTEMS_HOME

# make sure server.hsconf is a symbolic link and in the correct place
if [ -e "/etc/outsystems/server.hsconf.rpmsave" -a ! -e "/etc/outsystems/server.hsconf" ]; then
        # Prevent #237436
        mv /etc/outsystems/server.hsconf.rpmsave /etc/outsystems/server.hsconf
fi
if [ -e "$OUTSYSTEMS_HOME/server.hsconf" -a ! -h "$OUTSYSTEMS_HOME/server.hsconf" ]; then
        # move server.hsconf to /etc/outsystems
        mv "$OUTSYSTEMS_HOME/server.hsconf" /etc/outsystems
fi
if [ ! -e "$OUTSYSTEMS_HOME/server.hsconf" -a ! -h "$OUTSYSTEMS_HOME/server.hsconf" ]; then
        # server.hsconf link was deleted accidentally?
        ln -s /etc/outsystems/server.hsconf "$OUTSYSTEMS_HOME/server.hsconf"
fi
if [ -e /etc/outsystems/server.hsconf ]; then
        chown outsystems:outsystems /etc/outsystems/server.hsconf
fi

# fix java preferences (prevents warnings in the first run of the configuration tool)
JAVA_SYSTEM_PREFS_DIR=/etc/.java/.systemPrefs

mkdir -p $JAVA_SYSTEM_PREFS_DIR
chmod 755 /etc/.java $JAVA_SYSTEM_PREFS_DIR

if [ ! -e "$JAVA_SYSTEM_PREFS_DIR/.system.lock" ]; then
        touch $JAVA_SYSTEM_PREFS_DIR/.system.lock
fi

if [ ! -e "$JAVA_SYSTEM_PREFS_DIR/.systemRootModFile" ]; then
        touch $JAVA_SYSTEM_PREFS_DIR/.systemRootModFile
fi

chmod 644 $JAVA_SYSTEM_PREFS_DIR/.system*
chown root:root $JAVA_SYSTEM_PREFS_DIR/.system*
mkdir -p $JAVA_SYSTEM_PREFS_DIR/outsystems
chmod 755 $JAVA_SYSTEM_PREFS_DIR/outsystems
chown -R outsystems:outsystems $JAVA_SYSTEM_PREFS_DIR/outsystems
chmod 644 $JAVA_SYSTEM_PREFS_DIR/outsystems/*

LOG4J_CONFIGURATION="/etc/outsystems/configurationtool.properties"
LOGFILENAME=""
if [ -e $LOG4J_CONFIGURATION ]; then
	VALUEFROMFILE=`sed '/^\#/d' $LOG4J_CONFIGURATION | grep 'log4j.appender.ConfigurationTool.File'  | tail -n 1 | cut -d "=" -f2-`
	if [ ! -z "$VALUEFROMFILE" ]; then 
		LOGFILENAME="$OUTSYSTEMS_HOME/$VALUEFROMFILE"
		touch $LOGFILENAME
		chown outsystems:outsystems $LOGFILENAME
	fi
fi

echo "Initializing..."

source $OUTSYSTEMS_HOME/common-connectors.sh

# Upgrade cleanup from pre-5.0 code
if [ -e "$OUTSYSTEMS_HOME/running/customHandlers.ear" ]; then
	rm "$OUTSYSTEMS_HOME/running/customHandlers.ear" >/dev/null 2>&1
fi

CLASSPATH="$COMMON_CLASSPATH:$OUTSYSTEMS_HOME/configurationtool.jar:$OUTSYSTEMS_HOME/outsystems.activation.jar"
COMMON_JAVA_OPTS="-Djava.security.policy=/etc/outsystems/configurationtool.policy -Dlog4j.configuration=file:$LOG4J_CONFIGURATION"
SUDO_OUTSYSTEMS="sudo -E -u outsystems LANG=en_US.UTF-8 JAVA_HOME=$JAVA_HOME OUTSYSTEMS_HOME=$OUTSYSTEMS_HOME ANT_LIBS=$ANT_LIBS"

# check cryptographic strength before running Configuration Tool
if ! $SUDO_OUTSYSTEMS $JAVA_HOME/bin/java $COMMON_JAVA_OPTS -classpath $CLASSPATH outsystems.hubedition.configurationtool.CheckCryptographicStrength; then
        if [ -f "$OUTSYSTEMS_HOME/jce_policy-6.zip" -a -r "$OUTSYSTEMS_HOME/jce_policy-6.zip" ]; then
                JCEPOLICY_FOUND="y"
        else
                JCEPOLICY_FOUND="n"
        fi

        while [ "$JCEPOLICY_FOUND" != "y" ]; do
                echo
                echo "For OutSystems Platform to work, you need to install Java Cryptography Extension (JCE) with unlimited strength."
                echo "Please, be advised, that in order to install JCE you must accept and comply with its license terms and conditions."
                echo "Please download it from http://www.oracle.com/technetwork/java/javase/downloads/jce-6-download-429243.html and save the jce_policy-6.zip file to $OUTSYSTEMS_HOME"
                echo "Press ENTER to continue..."
                read tmp

                if [ -f "$OUTSYSTEMS_HOME/jce_policy-6.zip" -a -r "$OUTSYSTEMS_HOME/jce_policy-6.zip" ]; then
                        JCEPOLICY_FOUND="y"
                fi
        done

        JDK_16="/etc/alternatives/java_sdk_1.6.0"
        JRE_16="$(readlink /etc/alternatives/jre_1.6.0)"
        JCE_PATH="/usr/lib/jvm-private/java-1.6.0-sun.x86_64/jce"

        if [ "$JAVA_HOME" = "$JDK_16" -a -e "$JCE_PATH" -a -e "$JRE_16" -a -e "$JDK_16" ]; then
                # Install via update-alternatives
                echo "Registering JCE libraries in $JCE_PATH/unlimited-strength/"
                unzip -o -q -j "$OUTSYSTEMS_HOME/jce_policy-6.zip" 'jce/*.jar' -d "$JCE_PATH/unlimited-strength/"
                update-alternatives --install "$JRE_16/lib/security/local_policy.jar" jce_1.6.0_sun_local_policy.x86_64 "$JCE_PATH/unlimited-strength/local_policy.jar" 160999 --slave "$JRE_16/lib/security/US_export_policy.jar" jce_1.6.0_sun_us_export_policy.x86_64 "$JCE_PATH/unlimited-strength/US_export_policy.jar"
        else
                # Install to jre/lib/security folder
                echo "Registering JCE libraries in $JAVA_HOME/jre/lib/security/"
                unzip -o -q -j "$OUTSYSTEMS_HOME/jce_policy-6.zip" 'jce/*.jar' -d "$JAVA_HOME/jre/lib/security/"
        fi
fi

JAVA_OPTS="$COMMON_JAVA_OPTS"
CUSTOM_SH="/etc/outsystems/configurationtool.conf"
if [ -r $CUSTOM_SH ]; then
    . $CUSTOM_SH
fi

# PLEASE DO NOT REMOVE. this FIRST_INSTALL variable is used in connectors. PLEASE DO NOT REMOVE
if $SUDO_OUTSYSTEMS $JAVA_HOME/bin/java $COMMON_JAVA_OPTS -classpath $CLASSPATH outsystems.hubedition.configurationtool.IsFirstInstall; then
        FIRST_INSTALL=y
else
        FIRST_INSTALL=n
fi

if ! $SUDO_OUTSYSTEMS $JAVA_HOME/bin/java $JAVA_OPTS -classpath $CLASSPATH outsystems.hubedition.configurationtool.LicenseAgreement "$@"; then
        echo
        exit
fi

processAvailableConnectors

if ! $SUDO_OUTSYSTEMS $JAVA_HOME/bin/java $JAVA_OPTS -classpath $CLASSPATH outsystems.hubedition.configurationtool.ConfigurationTool "$@"; then
        echo
        exit
fi

silentmode=n
scinstallmode=n
until [ -z "$1" ]; do
        lower=`echo $1 | tr '[:upper:]' '[:lower:]'`
        if [ "$lower" = "/silent" ]; then
                silentmode=y
        fi
        if [ "$lower" = "/scinstall" ]; then
                scinstallmode=y
        fi
        shift
done

# check iptables port forwarding
function isPortForwarded() {
        CHAIN=$1
        IP=$2
        FROM=$3
        TO=$4        
        return $(/sbin/iptables -t nat -L $CHAIN -n 2>/dev/null | grep "tcp.*${IP//./\.}.*dpt:$FROM.*redir ports $TO" 1>/dev/null)
}

function isPortForwardedByInterface() {
        CHAIN=$1
        INTERFACE=$2
        FROM=$3
        TO=$4
        return $(/sbin/iptables -t nat -L $CHAIN -nv 2>/dev/null | grep "tcp.*$INTERFACE.*dpt:$FROM.*redir ports $TO" 1>/dev/null)
}

function isPortAccepting() {
        CHAIN=$1
        PORT=$2
        return $(/sbin/iptables -t filter -L $CHAIN -n 2>/dev/null | tail -n +3 | grep "tcp.*dpt:$PORT state NEW" 1>/dev/null)
}

iptablesFeedback(){
    if [ "$IPTABLES_FEEDBACK" != "y" ]; then
        echo Configuring iptables redirection rules...;
        IPTABLES_FEEDBACK="y";
    fi
}


if ! isPortAccepting INPUT 8080; then
        iptablesFeedback
        /sbin/iptables -t filter -I INPUT -p tcp --dport 8080 -m state --state NEW -j ACCEPT -m comment --comment "outsystems"
        /sbin/iptables-save > /etc/sysconfig/iptables
fi

if ! isPortAccepting INPUT 8443; then
        iptablesFeedback
        /sbin/iptables -t filter -I INPUT -p tcp --dport 8443 -m state --state NEW -j ACCEPT -m comment --comment "outsystems"
        /sbin/iptables-save > /etc/sysconfig/iptables
fi

if ! isPortForwarded PREROUTING 0.0.0.0 80 8080; then
        iptablesFeedback
        /sbin/iptables -t nat -I PREROUTING -p tcp --dport 80 -j REDIRECT --to-port 8080 -m comment --comment "outsystems"
        /sbin/iptables-save > /etc/sysconfig/iptables
fi

if ! isPortForwarded PREROUTING 0.0.0.0 443 8443; then
        iptablesFeedback
        /sbin/iptables -t nat -I PREROUTING -p tcp --dport 443 -j REDIRECT --to-port 8443 -m comment --comment "outsystems"
        /sbin/iptables-save > /etc/sysconfig/iptables
fi

for INTERFACE in `$SUDO_OUTSYSTEMS $JAVA_HOME/bin/java $COMMON_JAVA_OPTS -classpath $CLASSPATH outsystems.hubedition.configurationtool.PrintNetworkInterfaces`; do
        if ! isPortForwardedByInterface OUTPUT $INTERFACE 80 8080; then
                iptablesFeedback
                /sbin/iptables -t nat -I OUTPUT -o $INTERFACE -p tcp --dport 80 -j REDIRECT --to-port 8080 -m addrtype --dst-type LOCAL -m comment --comment "outsystems"
                /sbin/iptables-save > /etc/sysconfig/iptables
        fi

        if ! isPortForwardedByInterface OUTPUT $INTERFACE 443 8443; then
                iptablesFeedback
                /sbin/iptables -t nat -I OUTPUT -o $INTERFACE -p tcp --dport 443 -j REDIRECT --to-port 8443 -m addrtype --dst-type LOCAL -m comment --comment "outsystems"
                /sbin/iptables-save > /etc/sysconfig/iptables
        fi
done

if askYesNo "$(isApplicationServerRestartNeeded)" "The Application Server needs to be restarted. Restart now?"; then
	activeConnectorsStopConnectorRelatedServices
fi

# set ISFRONTEND variable so that it can be used if needed
if [ "$DEPLOYER" = "ENABLED" ]; then
	ISFRONTEND="y"
else
	ISFRONTEND="n"
fi

activeConnectorsStartConnectorRelatedServicesIfNeeded

if (/etc/init.d/outsystems status scheduler | grep RUNNING 1>/dev/null); then	
	START_SCHEDULER=y
else
	if [ "$SCHEDULER" = "ENABLED" ]; then
		echo -e "\n\e[49;33;3m$(printableName SCHEDULER) is currently stopped: Timers, Processes, and Emails won't be executed/sent.\e[m\n"
		if askYesNo "y" "Start this service now?"; then
			START_SCHEDULER=y
		else
			START_SCHEDULER=n
		fi
	else
		START_SCHEDULER=n
	fi
fi

if askYesNo "y" "\nDo you want to run Service Center installation?"; then
    DO_SCINSTALL="y"
else
    DO_SCINSTALL="n"
fi

echo -e "\n===== Restarting OutSystems Services =====\n"
/etc/init.d/outsystems stop rmi
/etc/init.d/outsystems stop controller
/etc/init.d/outsystems stop deployer
/etc/init.d/outsystems stop logserver
/etc/init.d/outsystems stop scheduler


/etc/init.d/outsystems start rmi
/etc/init.d/outsystems start controller
/etc/init.d/outsystems start deployer
/etc/init.d/outsystems start logserver

if [ "$DO_SCINSTALL" = "y" ]; then
	$OUTSYSTEMS_HOME/scinstall.sh
fi

if [ "$SCHEDULER" = "ENABLED" ] ; then
    if [ "$START_SCHEDULER" = "y" ] ; then
        /etc/init.d/outsystems start scheduler
    else
        log_warning_msg "$(printableName SCHEDULER) was not started."
    fi
fi



